package org.processmining.plugins.test.woflan;

import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.Connection;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.util.HTMLToString;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.plugins.petrinet.behavioralanalysis.woflan.WoflanConnection;
import org.processmining.plugins.petrinet.behavioralanalysis.woflan.WoflanDiagnosis;

public class WoflanTest implements HTMLToString {

	private class WoflanTestData {

		private String fileName;
		private Boolean bounded;
		private Boolean notDead;
		private Boolean live;

		protected WoflanTestData(String fileName, Boolean bounded,
				Boolean notdead, Boolean live) {
			this.fileName = fileName;
			this.bounded = bounded;
			this.notDead = notdead;
			this.live = live;
		}

		protected String getFileName() {
			return fileName;
		}

		protected Boolean isBounded() {
			return bounded;
		}

		protected Boolean isNotDead() {
			return notDead;
		}

		protected Boolean isLive() {
			return live;
		}
	}

	private int nofTests = 11;
	private WoflanTestData referenceData[] = new WoflanTestData[nofTests];
	private WoflanTestData testData[] = new WoflanTestData[nofTests];

	private void initTest() {
		referenceData[0] = new WoflanTestData(
				"D:\\Projects\\ProM\\Test\\TPN\\Woflan use case 1.tpn", null,
				null, null);
		referenceData[1] = new WoflanTestData(
				"D:\\Projects\\ProM\\Test\\TPN\\Woflan use case 2.tpn", null,
				null, null);
		referenceData[2] = new WoflanTestData(
				"D:\\Projects\\ProM\\Test\\TPN\\Woflan use case 3.tpn", true,
				true, true);
		referenceData[3] = new WoflanTestData(
				"D:\\Projects\\ProM\\Test\\TPN\\Woflan use case 4.tpn", true,
				true, false);
		referenceData[4] = new WoflanTestData(
				"D:\\Projects\\ProM\\Test\\TPN\\Woflan use case 5.tpn", true,
				false, null);
		referenceData[5] = new WoflanTestData(
				"D:\\Projects\\ProM\\Test\\TPN\\Woflan use case 6.tpn", true,
				true, true);
		referenceData[6] = new WoflanTestData(
				"D:\\Projects\\ProM\\Test\\TPN\\Woflan use case 7.tpn", true,
				true, false);
		referenceData[7] = new WoflanTestData(
				"D:\\Projects\\ProM\\Test\\TPN\\Woflan use case 8.tpn", true,
				true, false);
		referenceData[8] = new WoflanTestData(
				"D:\\Projects\\ProM\\Test\\TPN\\Woflan use case 9.tpn", false,
				null, null);
		referenceData[9] = new WoflanTestData(
				"D:\\Projects\\ProM\\Test\\TPN\\Woflan use case 10.tpn", true,
				false, null);
		referenceData[10] = new WoflanTestData(
				"D:\\Projects\\ProM\\Test\\TPN\\Woflan use case 11.tpn", false,
				null, null);
	}

	@Plugin(name = "WoflanTest", returnLabels = { "Woflan D:\\Projects\\ProM\\Test\\TPN\\Woflan use case Test" }, returnTypes = { WoflanTest.class }, parameterLabels = {}, userAccessible = true)
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	public WoflanTest runTest(PluginContext context) throws Exception {
		initTest();
		for (int i = 0; i < nofTests; i++) {
			String fileName = referenceData[i].getFileName();
			Petrinet net = context.tryToFindOrConstructFirstNamedObject(
					Petrinet.class, "Open TPN file", Connection.class, "",
					fileName);
			WoflanDiagnosis diagnosis = context
					.tryToFindOrConstructFirstNamedObject(
							WoflanDiagnosis.class, "Woflan",
							WoflanConnection.class, WoflanConnection.DIAGNOSIS,
							net);
			testData[i] = new WoflanTestData(fileName, diagnosis.isBounded(), diagnosis.isNotDead(), diagnosis.isLive());
		}
		return this;
	}

	@Override
	public String toHTMLString(boolean includeHTMLTags) {
		StringBuffer headBuffer = new StringBuffer();
		StringBuffer dataBuffer = new StringBuffer();
		boolean testOK = true;
		dataBuffer.append("<dl>");
		for (int i = 0; i < nofTests; i++) {
			dataBuffer.append("<dt>");
			dataBuffer.append(referenceData[i].getFileName());
			dataBuffer.append("</dt><dd><ul>");
			if (testData[i].isBounded() == referenceData[i].isBounded()) {
				dataBuffer.append("<li color=\"green\">Boundedness: OK (");
				dataBuffer.append(testData[i].isBounded());
				dataBuffer.append(")</li>");
			} else {
				dataBuffer.append("<li color=\"red\">Boundedness: Not OK (expected ");
				dataBuffer.append(referenceData[i].isBounded());
				dataBuffer.append(", got ");
				dataBuffer.append(testData[i].isBounded());
				dataBuffer.append(")</li>");
				testOK = false;
			}
			if (testData[i].isNotDead() == referenceData[i].isNotDead()) {
				dataBuffer.append("<li color=\"green\">No dead transitions: OK (");
				dataBuffer.append(testData[i].isNotDead());
				dataBuffer.append(")</li>");
			} else {
				dataBuffer.append("<li color=\"red\">No dead transitions: Not OK (expected ");
				dataBuffer.append(referenceData[i].isNotDead());
				dataBuffer.append(", got ");
				dataBuffer.append(testData[i].isNotDead());
				dataBuffer.append(")</li>");
				testOK = false;
			}
			if (testData[i].isLive() == referenceData[i].isLive()) {
				dataBuffer.append("<li color=\"green\">Boundedness: OK (");
				dataBuffer.append(testData[i].isLive());
				dataBuffer.append(")</li>");
			} else {
				dataBuffer.append("<li color=\"red\">Boundnedness: Not OK (expected ");
				dataBuffer.append(referenceData[i].isLive());
				dataBuffer.append(", got ");
				dataBuffer.append(testData[i].isLive());
				dataBuffer.append(")</li>");
				testOK = false;
			}
			dataBuffer.append("</ul></dd>");
		}
		dataBuffer.append("</dl>");
		if (testOK) {
			headBuffer.append("<h1>Woflan Test <font color=\"green\">OK</font></h1>");
		} else {
			headBuffer.append("<h1>Woflan Test <font color=\"red\">not OK</font></h1>");
		}
		headBuffer.append(dataBuffer.toString());
		return headBuffer.toString();
	}
}
